/*
 ============================================================================
 Name		 : Types.cpp
 Copyright   : Symbian Academy, 2007
 Description : Exercise: Symbian OS Types and Declarations
 ============================================================================
 */

//  Include Files  
#include <e32base.h>
#include <e32std.h>
#include <e32cons.h>			// Console

#include <e32math.h>			// Randomizer in Math Library

// Function prototype (C-style).
LOCAL_C void MainL();

GLDEF_C TInt E32Main()
	{

	// Run application code inside TRAP harness, wait keypress when terminated
	TRAPD(mainError, MainL())
	;

	_LIT(KTextFailed, "Application Error, code: ");
	__ASSERT_ALWAYS(!mainError, User::Panic(KTextFailed, mainError));

	return KErrNone;
	}

LOCAL_C void MainL()
	{
	// ------------------------------------------------------------------
	// Initialization
	// ------------------------------------------------------------------
	// ** Create the console window for text output

	// Constant text, used for the title of the console window
	_LIT(KTextConsoleTitle, "Console - Types");

	// Create the output console.
	// This is an example on how to create a C type class.
	// CConsoleBase is derived from CBase and is instantiated on the heap.
	// The special creation with NewL will be explained in the memory
	// handling module.
	CConsoleBase* console = Console::NewL(KTextConsoleTitle, TSize(KConsFullScreen,KConsFullScreen));

	// ------------------------------------------------------------------
	// Using TInt, text output
	// ------------------------------------------------------------------
	// ** Some basic experiments with TInt variables and text output.

	// Take a look at the following example, which defines:
	// - a constant TInt variable called numExample
	// - a constant text definition called KWelcomeText, with the placeholder
	//   %d for a decimal variable
	// Finally, the example outputs the formatted string to the console.
	const TInt numExample = 1;
	_LIT(KWelcomeText, "Welcome to example %d!\n");
	console->Printf (KWelcomeText, numExample);

	// Edit 1: Create a constant text using the macro _LIT called KCountText.
	//         It will be used to output the text "Count: 1\n", where 1 
	//         should be dynamically replaced by a decimal value.
	//         What _LIT really does will be handled in a later module;
	//         for now, define it like in the example above.
	

	// Edit 2: Define a constant TInt variable called maxCount with the
	//         value of 5.
	

	// Edit 3: Create a small for-loop, which should count from 1 to maxCount.
	//         Use a TInt called i for counting. In the loop,
	//         print the constant text to the console, formated with i.
	

	// ------------------------------------------------------------------
	// Random numbers, TBool and TChar
	// ------------------------------------------------------------------
	// ** Lets the user guess a random number
	_LIT(KGuessInstructions, "Guess a random num [0..9]\n");
	console->Printf(KGuessInstructions);
	
	// Some text definitions for keeping the user informed
	_LIT(KYourGuess, "Your guess: %d\n");
	_LIT(KWrongGuess, "Guess again...\n");
	_LIT(KCorrectGuess, "Correct!\n");

	// Edit 4: Get a random number using Math::Random() and store it
	//         in a variable called random of type TUint32 (as
	//         returned by the Random()-function).
	//         Limit it to 0..9, e.g. using the mod operator (%).
	

	// Edit 5: Create a TBool variable called continueLoop and
	//         initialize it to ETrue.
	

	// Edit 6: Create a TChar variable called keyChar.
	

	do
		{
		// Edit 7: Use the console->Getch() method to query a
		//         key and store the results in keyChar.
		

		// Edit 8: Define a TInt variable called curNum.
		//         Use the GetNumericValue()-method provided
		//         by TChar to convert keyChar to a TInt.
		

		// Edit 9: GetNumericValue() will return a negative
		//         number in case of an incorrect input.
		//         Create an if statement to only execute
		//         the winner-check if curNum is positive.
		
			{
			// Edit 10: Print the current guess (curNum) using 
			//         the KYourGuess literal defined above.
			
			
			// Edit 11: Check if the entered number (curNum)
			//          equals the random number (random).
			//          .. if yes: set continueLoop to EFalse
			//             and print the KCorrectGuess-message.
			//          .. if no: print the KWrongGuess-message.
			
			}
		}
	// Edit 12: Modify the while-statement to: while (continueLoop);
	while (EFalse);

	// ------------------------------------------------------------------
	// R Classes
	// ------------------------------------------------------------------
	// ** How to use an R class
	// The RBuf-class owns string data on the heap and manages it automatically.
	_LIT(KDynamicText, "Dynamic Text\n");

	// Edit 13: Define a new local variable of type RBuf with the name dynText.
	//          Even though the class will manage heap data, do not instanciate
	//          it as you would for example do for a C type class 
	//          (-> no new-operator)
	

	// Edit 14: Allocate memory and copy the static text KDynamicText
	//          into the dynText variable. Use the CreateL(...)-function of 
	//          the RBuf.
	

	// Edit 15: Convert the text to uppercase with the UpperCase()-function
	//          provided by the TDes base class of RBuf.
	

	// Edit 16: Uncomment the following line to print the text
	//console->Printf(dynText);

	// Edit 17: Free the memory for the RBuf by calling Close() - do not
	//          use delete for R type variables!
	

	// ------------------------------------------------------------------
	// Cleanup
	// ------------------------------------------------------------------

	// Wait for the space key before exiting.
	// Generally better than accepting any key to prevent immediate shutdown
	// if a key has still been in the queue when the app. started.
	_LIT(KTextPressSpace, "\n[press space to exit]\n");
	console->Printf (KTextPressSpace);
	while (console->Getch() != EKeySpace)
		{}

	delete console;
	}

